@@ -1,3 +1,4 @@
+import json
 import uuid
 import warnings
 from urllib.parse import parse_qsl, urlparse
@@ -5,19 +6,19 @@ from urllib.parse import parse_qsl, urlparse
 from django import forms
 from django.contrib import admin
 from django.contrib.admin.helpers import AdminForm
-from django.contrib.admin.utils import get_deleted_objects
+from django.contrib.admin.utils import flatten_fieldsets, get_deleted_objects
 from django.core.exceptions import PermissionDenied
-from django.db import transaction
+from django.db import models, transaction
 from django.http import (
     HttpResponse,
     HttpResponseBadRequest,
     HttpResponseForbidden,
     HttpResponseNotFound,
-    HttpResponseRedirect,
 )
-from django.shortcuts import get_list_or_404, get_object_or_404, render
+from django.shortcuts import get_list_or_404, get_object_or_404
 from django.template.response import TemplateResponse
-from django.urls import re_path
+from django.urls import include, re_path
+from django.utils.datastructures import MultiValueDict
 from django.utils.decorators import method_decorator
 from django.utils.encoding import force_str
 from django.utils.html import conditional_escape
@@ -32,18 +33,19 @@ from cms.exceptions import PluginLimitReached
 from cms.models.placeholdermodel import Placeholder
 from cms.models.placeholderpluginmodel import PlaceholderReference
 from cms.models.pluginmodel import CMSPlugin
+from cms.plugin_base import CMSPluginBase
 from cms.plugin_pool import plugin_pool
 from cms.signals import post_placeholder_operation, pre_placeholder_operation
-from cms.toolbar.utils import get_plugin_tree_as_json
+from cms.toolbar.utils import create_child_plugin_references, get_plugin_content, get_plugin_tree
 from cms.utils import get_current_site
-from cms.utils.compat.warnings import RemovedInDjangoCMS60Warning
+from cms.utils.compat.warnings import RemovedInDjangoCMS51Warning
 from cms.utils.conf import get_cms_setting
 from cms.utils.i18n import get_language_code, get_language_list
 from cms.utils.plugins import (
     copy_plugins_to_placeholder,
+    downcast_plugins,
     has_reached_plugin_limit,
 )
-from cms.utils.urlutils import admin_reverse
 from cms.views import (
     render_object_edit,
     render_object_preview,
@@ -79,72 +81,48 @@ def _instance_overrides_method(base, instance, method_name):
     return unbound_method != bound_method
 
 
-class FrontendEditableAdminMixin:
+class BaseEditableAdminMixin:
     """
-    Adding ``FrontendEditableAdminMixin`` to  models admin class allows to open that admin
-    in the frontend by double-clicking on fields rendered with the ``render_model`` template
-    tag.
+    Base class for FrontendEditableAdminMixin to be reused by
+    PlaceholderAdmin
     """
-    frontend_editable_fields = []
-
-    def get_urls(self):
-        """
-        Register the url for the single field edit view
-        """
-        info = "%s_%s" % (self.model._meta.app_label, self.model._meta.model_name)
-        def pat(regex, fn):
-            return re_path(regex, self.admin_site.admin_view(fn), name="%s_%s" % (info, fn.__name__))
-        url_patterns = [
-            pat(r'edit-field/(%s)/([a-z\-]+)/$' % SLUG_REGEXP, self.edit_field),
-        ]
-        return url_patterns + super().get_urls()
-
-    def _get_object_for_single_field(self, object_id, language):
-        # Quick and dirty way to retrieve objects for django-hvad
-        # Cleaner implementation will extend this method in a child mixin
-        try:
-            # First see if the model uses the admin manager pattern from cms.models.manager.ContentAdminManager
-            manager = self.model.admin_manager
-        except AttributeError:
-            # If not, use the default manager
-            manager = self.model.objects
-        try:
-            return manager.language(language).get(pk=object_id)
-        except AttributeError:
-            return manager.get(pk=object_id)
-
+    @xframe_options_sameorigin
     def edit_field(self, request, object_id, language):
+        """Endpoint which manages frontend-editable fields"""
         obj = self._get_object_for_single_field(object_id, language)
         opts = obj.__class__._meta
         saved_successfully = False
         cancel_clicked = request.POST.get("_cancel", False)
-        raw_fields = request.GET.get("edit_fields")
-        fields = [field for field in raw_fields.split(",") if field in self.frontend_editable_fields]
+        raw_fields = request.GET.get("edit_fields", "")
+        admin_obj = self._get_model_admin(obj)
+        allowed_fields = getattr(admin_obj, "frontend_editable_fields", [])
+        fields = [field for field in raw_fields.split(",") if field in allowed_fields]
         if not fields:
             context = {
                 'opts': opts,
                 'message': _("Field %s not found") % raw_fields
             }
-            return render(request, 'admin/cms/page/plugin/error_form.html', context)
-        if not request.user.has_perm(f"{self.model._meta.app_label}.change_{self.model._meta.model_name}"):
+            return TemplateResponse(request, 'admin/cms/page/plugin/error_form.html', context)
+        if not request.user.has_perm(f"{admin_obj.model._meta.app_label}.change_{admin_obj.model._meta.model_name}"):
             context = {
                 'opts': opts,
                 'message': _("You do not have permission to edit this item")
             }
-            return render(request, 'admin/cms/page/plugin/error_form.html', context)
+            return TemplateResponse(request, 'admin/cms/page/plugin/error_form.html', context)
             # Dynamically creates the form class with only `field_name` field
         # enabled
-        form_class = self.get_form(request, obj, fields=fields)
+        form_class = admin_obj.get_form(request, obj, fields=fields)
         if not cancel_clicked and request.method == 'POST':
             form = form_class(instance=obj, data=request.POST)
             if form.is_valid():
-                form.save()
+                new_object = form.save(commit=False)
+                admin_obj.save_model(request, new_object, form, change=True)  # Call save model like the admin does
                 saved_successfully = True
         else:
             form = form_class(instance=obj)
         admin_form = AdminForm(form, fieldsets=[(None, {'fields': fields})], prepopulated_fields={},
                                model_admin=self)
-        media = self.media + admin_form.media
+        media = admin_obj.media + admin_form.media
         context = {
             'CMS_MEDIA_URL': get_cms_setting('MEDIA_URL'),
             'title': opts.verbose_name,
@@ -162,13 +140,51 @@ class FrontendEditableAdminMixin:
         }
         if cancel_clicked:
             # cancel button was clicked
-            context.update({
-                'cancel': True,
-            })
-            return render(request, 'admin/cms/page/plugin/confirm_form.html', context)
-        if not cancel_clicked and request.method == 'POST' and saved_successfully:
-            return render(request, 'admin/cms/page/plugin/confirm_form.html', context)
-        return render(request, 'admin/cms/page/plugin/change_form.html', context)
+            context["cancel"] = True
+            return TemplateResponse(request, 'admin/cms/page/plugin/confirm_form.html', context)
+        if not cancel_clicked and request.method == 'POST' and saved_successfully and isinstance(admin_obj, CMSPluginBase):
+            # Update the structure board by populating the data bridge
+            return admin_obj.render_close_frame(request, obj, action="change")
+        return TemplateResponse(request, 'admin/cms/page/plugin/change_form.html', context)
+
+
+class FrontendEditableAdminMixin(BaseEditableAdminMixin):
+    """
+    Adding ``FrontendEditableAdminMixin`` to  models admin class allows to open that admin
+    in the frontend by double-clicking on fields rendered with the ``render_model`` template
+    tag.
+    """
+    def get_urls(self) -> list[str]:
+        """
+        Register the url for the edit field view
+        """
+        info = f"{self.model._meta.app_label}_{self.model._meta.model_name}"
+
+        def pat(regex, fn):
+            return re_path(regex, self.admin_site.admin_view(fn), name=f"{info}_{fn.__name__}")
+        url_patterns = [
+            pat(r'edit-field/(%s)/([a-z\-]+)/$' % SLUG_REGEXP, self.edit_field),
+        ]
+        return url_patterns + super().get_urls()
+
+    def _get_model_admin(self, obj: models.Model) -> admin.ModelAdmin:
+        # FrontendEditableAdminMixin needs to be added to the model's model admin class.
+        # Hence, the relevant admin is the model admin itself.
+        return self
+
+    def _get_object_for_single_field(self, object_id: int, language: str) -> models.Model:
+        # Quick and dirty way to retrieve objects for django-hvad
+        # Cleaner implementation will extend this method in a child mixin
+        try:
+            # First see if the model uses the admin manager pattern from cms.models.manager.ContentAdminManager
+            manager = self.model.admin_manager
+        except AttributeError:
+            # If not, use the default manager
+            manager = self.model.objects
+        try:
+            return manager.language(language).get(pk=object_id)
+        except AttributeError:
+            return manager.get(pk=object_id)
 
 
 class PlaceholderAdminMixinBase(forms.MediaDefiningClass):
@@ -180,7 +196,7 @@ class PlaceholderAdminMixinBase(forms.MediaDefiningClass):
             return super_new(cls, name, bases, attrs)
         warnings.warn(
             "PlaceholderAdminMixin is no longer needed and thus will be removed in django CMS 5.0",
-            RemovedInDjangoCMS60Warning,
+            RemovedInDjangoCMS51Warning,
             stacklevel=2,
         )
         return super_new(cls, name, bases, attrs)
@@ -195,7 +211,10 @@ class PlaceholderAdminMixin(metaclass=PlaceholderAdminMixinBase):
     pass
 
 
-class PlaceholderAdmin(admin.ModelAdmin):
+@admin.register(Placeholder)
+class PlaceholderAdmin(BaseEditableAdminMixin, admin.ModelAdmin):
+    """Placeholder admin manages placeholders and their plugins, as well as the preview, edit, and
+    structure endpoints."""
 
     def has_add_permission(self, request):
         # Placeholders are created by the system
@@ -219,28 +238,42 @@ class PlaceholderAdmin(admin.ModelAdmin):
         # but the admin's delete view is not available for placeholders.
         raise PermissionDenied
 
-    def get_urls(self):
+    def get_urls(self) -> list[str]:
         """
         Register the plugin specific urls (add/edit/copy/remove/move)
         """
-        info = "%s_%s" % (self.model._meta.app_label, self.model._meta.model_name)
+        info = f"{self.model._meta.app_label}_{self.model._meta.model_name}"
+
         def pat(regex, fn):
-            return re_path(regex, self.admin_site.admin_view(fn), name="%s_%s" % (info, fn.__name__))
+            return re_path(regex, self.admin_site.admin_view(fn), name=f"{info}_{fn.__name__}")
         url_patterns = [
+            re_path(r'^cms_wizard/', include('cms.wizards.urls')),
             pat(r'^copy-plugins/$', self.copy_plugins),
             pat(r'^add-plugin/$', self.add_plugin),
             pat(r'^edit-plugin/([0-9]+)/$', self.edit_plugin),
+            pat(r'^edit-field/([0-9]+)/([a-z\-]+)/$', self.edit_field),
             pat(r'^delete-plugin/([0-9]+)/$', self.delete_plugin),
             pat(r'^clear-placeholder/([0-9]+)/$', self.clear_placeholder),
             pat(r'^move-plugin/$', self.move_plugin),
             # Register object edit/structure/preview endpoints.
-            # pat(r'^object/(?P<content_type_id>\d+)/structure/(?P<object_id>.+)$', render_object_structure),
             pat(r'^object/([0-9]+)/edit/([0-9]+)/$', render_object_edit),
             pat(r'^object/([0-9]+)/structure/([0-9]+)/$', render_object_structure),
             pat(r'^object/([0-9]+)/preview/([0-9]+)/$', render_object_preview),
         ]
         return url_patterns
 
+    def _get_object_for_single_field(self, object_id: int, language: str) -> CMSPlugin:
+        # For BaseEditableAdminMixin: This (private) method retrieves the corresponding CMSPlugin and
+        # downcasts it to the appropriate plugin model. language is ignored. This provides the plugin for
+        # edit_field"""
+        plugin = get_object_or_404(CMSPlugin, pk=object_id)  # Returns a CMSPlugin instance
+        return plugin.get_bound_plugin()  # Returns the plugin model instance of the appropriate type
+
+    def _get_model_admin(self, obj: CMSPlugin) -> admin.ModelAdmin:
+        # For BaseEditableAdminMixin: This (private) method retrieves the model admin for the plugin model
+        # which is the plugin instance itself.
+        return obj.get_plugin_class_instance(admin=self.admin_site)
+
     def _get_operation_language(self, request):
         # Unfortunately the ?language GET query
         # has a special meaning on the CMS.
@@ -398,8 +431,31 @@ class PlaceholderAdmin(admin.ModelAdmin):
             'position': plugin_data['plugin_position'],
         }
 
-        response = plugin_instance.add_view(request)
+        if request.method == 'POST' and not plugin_class.show_add_form:
+            # If the plugin has show_add_form set to False,
+            # the post data is missing the initial values of the plugin form
+            # Get the fields, the form and the initial values from the plugin instance
+            # Replace the POST parameters by those initial values plus any concrete changes
+            # the form.
+            # TODO: Make this work with Text plugins which use ghost plugins and have to
+            # have show_add_form=True
+            fieldsets = plugin_instance.get_fieldsets(request, obj=None)
+            fields = flatten_fieldsets(fieldsets)
+            # Instantiate the add form for all fields
+            initial_form = plugin_instance.get_form(request, None, change=False, fields=fields)()
+            # Turn the initial values in a multi-value dict. In a multi-value dict each value is a list.
+            # Hence, if the initial value is not a list, it is turned into a list.
+            query_dict = MultiValueDict({
+                name: field.initial if isinstance(field.initial, (tuple, list)) else [field.initial]
+                for name, field in initial_form.fields.items()
+                if getattr(field, "initial", None) is not None and name not in request.POST
+            })
+            # Add the actual post parameters
+            query_dict.update(request.POST)
+            # Use the QueryDict as the POST data
+            request.POST = query_dict
 
+        response = plugin_instance.add_view(request)
         plugin = getattr(plugin_instance, 'saved_object', None)
 
         if plugin_instance._operation_token:
@@ -457,8 +513,8 @@ class PlaceholderAdmin(admin.ModelAdmin):
                 source_placeholder,
                 target_placeholder,
             )
-        data = get_plugin_tree_as_json(request, new_plugins)
-        return HttpResponse(data, content_type='application/json')
+        data = get_plugin_tree(request, new_plugins)
+        return HttpResponse(json.dumps(data), content_type='application/json')
 
     def _copy_plugin_to_clipboard(self, request, target_placeholder):
         source_language = request.POST['source_language']
@@ -635,6 +691,7 @@ class PlaceholderAdmin(admin.ModelAdmin):
         POST request with following parameters:
         - plugin_id
         - placeholder_id
+        - target_position (optional)
         - plugin_language (optional)
         - plugin_parent (optional)
         - plugin_order (array, optional)
@@ -660,6 +717,7 @@ class PlaceholderAdmin(admin.ModelAdmin):
             placeholder = None
 
         # The rest are optional
+        target_position = int(request.POST.get('target_position', "0"))
         parent_id = get_int(request.POST.get('plugin_parent', ""), None)
         target_language = request.POST['target_language']
         move_a_copy = request.POST.get('move_a_copy')
@@ -698,50 +756,63 @@ class PlaceholderAdmin(admin.ModelAdmin):
         else:
             target_parent = None
 
-        new_plugin = None
-        fetch_tree = False
+        old_parent = None
+        fetch_tree = True
 
         if move_a_copy and plugin.plugin_type == "PlaceholderPlugin":
+            fetch_tree = False
+            new_plugin = None
             new_plugins = self._paste_placeholder(
                 request,
                 plugin=plugin,
                 target_language=target_language,
                 target_placeholder=placeholder,
-                target_position=int(request.POST['target_position']),
+                target_position=target_position,
             )
         elif move_a_copy:
-            fetch_tree = True
             new_plugin = self._paste_plugin(
                 request,
                 plugin=plugin,
                 target_parent=target_parent,
                 target_language=target_language,
                 target_placeholder=placeholder,
-                target_position=int(request.POST['target_position']),
+                target_position=target_position,
             )
         elif move_to_clipboard:
+            old_parent = plugin.parent  # Previous parent needs content update, too
             new_plugin = self._cut_plugin(
                 request,
                 plugin=plugin,
                 target_language=target_language,
                 target_placeholder=placeholder,
             )
-            new_plugins = [new_plugin]
+            new_plugins = [plugin]
         else:
-            fetch_tree = True
+            old_parent = plugin.parent  # Previous parent needs content update, too
             new_plugin = self._move_plugin(
                 request,
                 plugin=plugin,
                 target_parent=target_parent,
-                target_position=int(request.POST['target_position']),
+                target_position=target_position,
                 target_placeholder=placeholder,
             )
 
         if new_plugin and fetch_tree:
             root = (new_plugin.parent or new_plugin)
             new_plugins = [root] + list(root.get_descendants())
-        data = get_plugin_tree_as_json(request, new_plugins)
-        return HttpResponse(data, content_type='application/json')
+        data = get_plugin_tree(request, new_plugins, target_plugin=new_plugins[0])
+        if old_parent and old_parent not in new_plugins and "content" in data:
+            # Update previous parent and its children
+            old_parent_plugins = list(downcast_plugins([old_parent] + list(old_parent.get_descendants()), select_placeholder=True))
+            create_child_plugin_references(old_parent_plugins)
+            data["content"] += get_plugin_content(request, old_parent_plugins[0])
+        # Pass the target_position
+        if "content" in data and len(data["content"]) > 0:
+            data["content"][0]["insert"] = new_plugins[0].pk == plugin.pk  # Insert the content (old_parent is always only updated)
+            if move_to_clipboard:
+                data["content"].pop(0)
+        data["source_placeholder_id"] = source_placeholder.pk
+        return HttpResponse(json.dumps(data), content_type='application/json')
 
     def _paste_plugin(self, request, plugin, target_language,
                       target_placeholder, target_position, target_parent=None):
@@ -769,7 +840,7 @@ class PlaceholderAdmin(admin.ModelAdmin):
             target_parent_id=target_parent_id,
         )
 
-        target_last_plugin = target_placeholder.get_last_plugin(plugin.language)
+        target_last_plugin = target_placeholder.get_last_plugin(target_language)
 
         if target_last_plugin:
             target_offset = target_last_plugin.position + len(plugins)
@@ -906,7 +977,6 @@ class PlaceholderAdmin(admin.ModelAdmin):
 
         # Refresh plugin to get new position values
         updated_plugin = plugin.reload()
-
         if target_placeholder:
             target_placeholder.clear_cache(language)
         source_placeholder.clear_cache(language)
@@ -997,10 +1067,9 @@ class PlaceholderAdmin(admin.ModelAdmin):
                 raise PermissionDenied(_("You do not have permission to delete this plugin"))
             obj_display = force_str(plugin)
             placeholder = plugin.placeholder
-            plugin_tree_order = placeholder.get_plugin_tree_order(
-                language=plugin.language,
-                parent_id=plugin.parent_id,
-            )
+            plugin_class_instance = plugin.get_plugin_class_instance()
+            plugin_parent = plugin.parent
+            plugin_tree_order = placeholder.get_plugin_tree_order(language=plugin.language)
 
             operation_token = self._send_pre_placeholder_operation(
                 request,
@@ -1023,7 +1092,11 @@ class PlaceholderAdmin(admin.ModelAdmin):
                 placeholder=placeholder,
                 tree_order=plugin_tree_order,
             )
-            return HttpResponseRedirect(admin_reverse('index', current_app=self.admin_site.name))
+            return plugin_class_instance.render_close_frame(
+                request,
+                plugin_parent,
+                action="delete",
+                extra_data=dict(deleted=True, plugin_id=plugin.pk))
 
         plugin_name = force_str(plugin.get_plugin_class().name)
 
@@ -1057,7 +1130,13 @@ class PlaceholderAdmin(admin.ModelAdmin):
             # There could be a case where a plugin has relationship to
             # an object the user does not have permission to delete.
             placeholder.clear(language)
-            return HttpResponseRedirect(admin_reverse('index', current_app=self.admin_site.name))
+            return TemplateResponse(request, "admin/cms/page/plugin/confirm_form.html", {
+                "data_bridge": {
+                    "action": "clear_placeholder",
+                    "deleted": True,
+                    "placeholder_id": placeholder.pk,
+                }
+            })
 
         if not self.has_clear_placeholder_permission(request, placeholder, language):
             message = _("You do not have permission to clear this placeholder")
@@ -1104,7 +1183,13 @@ class PlaceholderAdmin(admin.ModelAdmin):
                 plugins=plugins,
                 placeholder=placeholder,
             )
-            return HttpResponseRedirect(admin_reverse('index', current_app=self.admin_site.name))
+            return TemplateResponse(request, "admin/cms/page/plugin/confirm_form.html", {
+                "data_bridge": {
+                    "action": "clear_placeholder",
+                    "deleted": True,
+                    "placeholder_id": placeholder.pk,
+                }
+            })
 
         if perms_needed or protected:
             title = _("Cannot delete %(name)s") % {"name": obj_display}
@@ -1123,6 +1208,3 @@ class PlaceholderAdmin(admin.ModelAdmin):
         }
         request.current_app = self.admin_site.name
         return TemplateResponse(request, "admin/cms/page/plugin/delete_confirmation.html", context)
-
-
-admin.site.register(Placeholder, PlaceholderAdmin)
