@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 import re
 import typing
 from copy import copy
@@ -53,7 +55,7 @@ class ChangeListActionsMixin(metaclass=forms.MediaDefiningClass):
 
     def get_actions_list(
         self,
-    ) -> typing.List[typing.Callable[[models.Model, HttpRequest], str]]:
+    ) -> list[typing.Callable[[models.Model, HttpRequest], str]]:
         """Collect list actions from implemented methods and return as list. Make sure to call
         it's ``super()`` instance when overwriting::
 
@@ -99,7 +101,7 @@ class ChangeListActionsMixin(metaclass=forms.MediaDefiningClass):
 
     def get_list_display(
         self, request: HttpRequest
-    ) -> typing.Tuple[typing.Union[str, typing.Callable[[models.Model], str]], ...]:
+    ) -> tuple[str | typing.Callable[[models.Model], str], ...]:
         list_display = super().get_list_display(request)
         return tuple(
             self.get_admin_list_actions(request) if item == "admin_list_actions" else item for item in list_display
@@ -165,7 +167,7 @@ class GrouperChangeListBase(ChangeList):
 
     _extra_grouping_fields = []
 
-    def get_filters_params(self, params: typing.Optional[dict] = None):
+    def get_filters_params(self, params: dict | None = None):
         lookup_params = super().get_filters_params(params)
         for field in self._extra_grouping_fields:
             if field in lookup_params:
@@ -229,7 +231,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
     #: The name of the ``ForeignKey`` in the content model that points to the grouper instance. If not given
     #: it is assumed to be the snake case name of the grouper model class, e.g. ``"blog_post"`` for the
     #: ``"BlogPost"`` model.
-    grouper_field_name: typing.Optional[str] = None
+    grouper_field_name: str | None = None
     #: Indicates additional grouping fields such as ``"language"`` for example. Additional grouping fields create
     #: tabs in the change form and a dropdown menu in the change list view.
     #:
@@ -238,16 +240,16 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
     #:      All fields serving as extra grouping fields must be part of the admin's
     #:      :attr:`~django.contrib.admin.ModelAdmin.fieldsets` setting for ``GrouperModelAdmin`` to work properly.
     #:      In the change form the fields will be invisible.
-    extra_grouping_fields: typing.Tuple[str, ...] = ()
+    extra_grouping_fields: tuple[str, ...] = ()
     #: The content model class to be used. Defaults to the model class named like the grouper model class
     #: plus ``"Content"`` at the end from the same app as the grouper model class, e.g., ``BlogPostContent`` if
     #: the grouper is ``BlogPost``.
-    content_model: typing.Optional[models.Model] = None
+    content_model: models.Model | None = None
     #: Name of the inverse relation field giving the set of content models belonging to a grouper model. Defaults to
     #: the first field found as an inverse relation. If you have more than one inverse relation please make sure
     #: to specify this field. An example would be if the blog post content model contained a many-to-many
     #: relationship to the grouper model for, say, related blog posts.
-    content_related_field: typing.Optional[str] = None
+    content_related_field: str | None = None
 
     change_list_template = "admin/cms/grouper/change_list.html"
     change_form_template = "admin/cms/grouper/change_form.html"
@@ -307,8 +309,8 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
         for content_field in self.form._content_fields:
             if (
                 not hasattr(self, CONTENT_PREFIX + content_field)
-                and content_field != self.grouper_field_name  # noqa: W504
-                and content_field not in self.extra_grouping_fields  # noqa: W504
+                and content_field != self.grouper_field_name
+                and content_field not in self.extra_grouping_fields
             ):
                 if CONTENT_PREFIX + content_field in self.list_display:
                     # Identify content fields in list_display to annotate to queryset
@@ -345,7 +347,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
         self,
         obj: models.Model,
         field_name: str,
-        request: typing.Optional[HttpRequest] = None,
+        request: HttpRequest | None = None,
     ) -> typing.Any:
         """Retrieves the content of a field stored in the content model. If request is given extra
         grouping fields are processed before."""
@@ -403,16 +405,18 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
                 setattr(self, field, value)
 
     @property
-    def current_content_filters(self) -> typing.Dict[str, typing.Any]:
+    def current_content_filters(self) -> dict[str, typing.Any]:
         """Filters needed to get the correct content model instance"""
-        return {field: getattr(self, field, self.get_extra_grouping_field(field)) for field in self.extra_grouping_fields}
+        return {
+            field: getattr(self, field, self.get_extra_grouping_field(field)) for field in self.extra_grouping_fields
+        }
 
     def get_language(self) -> str:
         """Hook on how to get the current language. By default, if it is set as a
         property, use the property, otherwise let Django provide it."""
         return getattr(self, "language", get_language())
 
-    def get_language_tuple(self) -> typing.Tuple[typing.Tuple[str, str], ...]:
+    def get_language_tuple(self) -> tuple[tuple[str, str], ...]:
         """Hook on how to get all available languages for the language selector."""
         return get_language_tuple()
 
@@ -439,7 +443,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
     def changeform_view(
         self,
         request: HttpRequest,
-        object_id: typing.Optional[str] = None,
+        object_id: str | None = None,
         form_url: str = "",
         extra_context: dict = None,
     ) -> HttpResponse:
@@ -459,7 +463,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
         self,
         request: HttpRequest,
         object_id: str,
-        extra_context: typing.Optional[dict] = None,
+        extra_context: dict | None = None,
     ) -> HttpResponse:
         """Update grouping field properties for delete view"""
         self.get_grouping_from_request(request)
@@ -469,7 +473,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
         self,
         request: HttpRequest,
         object_id: str,
-        extra_context: typing.Optional[dict] = None,
+        extra_context: dict | None = None,
     ) -> HttpResponse:
         """Update grouping field properties for history view"""
         self.get_grouping_from_request(request)
@@ -492,9 +496,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
             preserved_filters["_changelist_filters"] = urlencode(grouping_filters)
         return urlencode(preserved_filters)
 
-    def get_extra_context(
-        self, request: HttpRequest, object_id: typing.Optional[str] = None
-    ) -> typing.Dict[str, typing.Any]:
+    def get_extra_context(self, request: HttpRequest, object_id: str | None = None) -> dict[str, typing.Any]:
         """Provide the grouping fields to the change view."""
         if object_id:
             # Instance provided? Get corresponding postconent
@@ -539,7 +541,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
         # TODO: Add context for other grouping fields to be shown as a dropdown
         return extra_context
 
-    def get_form(self, request: HttpRequest, obj: typing.Optional[models.Model] = None, **kwargs) -> type:
+    def get_form(self, request: HttpRequest, obj: models.Model | None = None, **kwargs) -> type:
         """Adds the language from the request to the form class"""
         form_class = super().get_form(request, obj, **kwargs)
         form_class._admin = self
@@ -616,7 +618,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
             ).latest_content()
         return self._content_qs_cache[obj]
 
-    def get_content_obj(self, obj: typing.Optional[models.Model]) -> typing.Optional[models.Model]:
+    def get_content_obj(self, obj: models.Model | None) -> models.Model | None:
         if obj is None or self._is_content_obj(obj):
             return obj
         else:
@@ -626,7 +628,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
                 )
             return self._content_obj_cache[obj]
 
-    def get_content_objects(self, obj: typing.Optional[models.Model]) -> models.QuerySet:
+    def get_content_objects(self, obj: models.Model | None) -> models.QuerySet:
         if obj is None:
             return None
         if self._is_content_obj(obj):
@@ -648,7 +650,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
             return getattr(obj, field_name)
         return obj
 
-    def view_on_site(self, obj: models.Model) -> typing.Optional[str]:
+    def view_on_site(self, obj: models.Model) -> str | None:
         # Adds the View on Site button to the admin
         content_obj = self.get_content_obj(obj)
         if content_obj:
@@ -656,7 +658,7 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
             return get_object_preview_url(content_obj, language=getattr(content_obj, "language", None))
         return None
 
-    def get_readonly_fields(self, request: HttpRequest, obj: typing.Optional[models.Model] = None):
+    def get_readonly_fields(self, request: HttpRequest, obj: models.Model | None = None):
         """Allow access to content fields to be controlled by a method "can_change_content":
         This allows versioned content to be protected if needed"""
         # First, get read-only fields for grouper
@@ -696,6 +698,50 @@ class GrouperModelAdmin(ChangeListActionsMixin, ModelAdmin):
             setattr(form._content_instance, self.grouper_field_name, obj)
             form._content_instance.save()
 
+    def get_search_fields(self, request):
+        """Return search fields for either grouper model or content model"""
+        content_search_fields = []
+        grouper_search_fields = []
+        for field_name in self.search_fields:
+            if field_name.startswith(CONTENT_PREFIX):
+                content_search_fields.append(field_name[len(CONTENT_PREFIX) :])
+            else:
+                grouper_search_fields.append(field_name)
+
+        if getattr(request, "_content_fields", False):
+            return content_search_fields
+
+        return grouper_search_fields
+
+    def get_search_results(self, request, queryset, search_term):
+        grouper_search_result, may_have_duplicate_grouper = super().get_search_results(request, queryset, search_term)
+
+        search_result_from_content, may_have_duplicate_content = self._get_content_search_result(
+            request, queryset, search_term
+        )
+
+        return grouper_search_result | search_result_from_content, (
+            may_have_duplicate_grouper & may_have_duplicate_content
+        )
+
+    def _get_content_search_result(self, request, queryset, search_term):
+        """Get search results from content model"""
+        try:
+            # Set flag on request object to get the content search fields. `get_search_results` will call
+            # `get_search_fields` to get the content search fields.
+            request._content_fields = True
+            content_queryset = self.content_model.admin_manager.all()
+            if self.get_search_fields(request):
+                content_search_result, __ = super().get_search_results(request, content_queryset, search_term)
+            else:
+                content_search_result = self.content_model.admin_manager.none()
+            search_result_from_content = queryset.filter(
+                id__in=content_search_result.values_list(f"{self.grouper_field_name}_id", flat=True)
+            )
+        finally:
+            request._content_fields = False
+        return search_result_from_content, False
+
 
 class _GrouperAdminFormMixin:
     _content_fields: list = []
@@ -738,7 +784,7 @@ class _GrouperAdminFormMixin:
         self.fields[CONTENT_PREFIX + self._admin.grouper_field_name].required = False
         self.update_labels(self._content_fields)
 
-    def update_labels(self, fields: typing.List[str]) -> None:
+    def update_labels(self, fields: list[str]) -> None:
         """Adds a language indicator to field labels"""
         if "language" in self._admin.extra_grouping_fields:
             language_dict = get_language_dict()
