@@ -159,6 +159,7 @@ class LogPageOperationsTests(CMSTestCase):
             endpoint = self.get_admin_url(Page, 'delete', page.pk)
             post_data = {'post': 'yes'}
 
+            self.assertEqual(0, LogEntry.objects.count())
             response = self.client.post(endpoint, post_data)
             # Test that the end point is valid
             self.assertEqual(response.status_code, 302)
@@ -380,7 +381,7 @@ class LogPlaceholderOperationsTests(CMSTestCase):
             data = {'post': True}
             response = self.client.post(endpoint, data)
             # Test that the end point is valid
-            self.assertEqual(response.status_code, 302)
+            self.assertContains(response, '<div class="success"></div>')
             # Test that the log count is correct
             self.assertEqual(1, LogEntry.objects.count())
 
@@ -563,7 +564,7 @@ class LogPlaceholderOperationsTests(CMSTestCase):
         with self.login_user_context(self._admin_user):
             response = self.client.post(endpoint, {'test': 0})
             # Test that the end point is valid
-            self.assertEqual(response.status_code, 302)
+            self.assertEqual(response.status_code, 200)
             # Test that the log count is correct
             self.assertEqual(1, LogEntry.objects.count())
 
